package netboxgo

import (
	"context"
	"fmt"
	"net/http"
	"net/url"
	"time"

	"github.com/gorilla/schema"
)

type DevicesService service

// Devices is used for Netbox dcim_device_list return struct
type Devices struct {
	Next     string   `json:"next"`
	Previous string   `json:"previous"`
	Results  []Device `json:"results"`
	Count    int      `json:"count"`
}

type Device struct {
	LastUpdated   time.Time   `json:"last_updated"`
	CustomFields  interface{} `json:"custom_fields,omitempty"`
	ConfigContext interface{} `json:"config_context"`
	Status        struct {
		Label string `json:"label"`
		Value string `json:"value"`
	} `json:"status"`
	Face struct {
		Label string `json:"label"`
		Value string `json:"value"`
	} `json:"face"`
	Name         string `json:"name"`
	AssetTag     string `json:"asset_tag"`
	Comments     string `json:"comments"`
	DisplayName  string `json:"display_name"`
	Created      string `json:"created"`
	Serial       string `json:"serial"`
	ParentDevice struct {
		URL         string `json:"url"`
		Name        string `json:"name"`
		DisplayName string `json:"display_name"`
		ID          int    `json:"id"`
	} `json:"parent_device"`
	Tenant struct {
		URL  string `json:"url"`
		Name string `json:"name"`
		Slug string `json:"slug"`
		ID   int    `json:"id"`
	} `json:"tenant"`
	Site struct {
		URL  string `json:"url"`
		Name string `json:"name"`
		Slug string `json:"slug"`
		ID   int    `json:"id"`
	} `json:"site"`
	Tags []struct {
		URL   string `json:"url"`
		Name  string `json:"name"`
		Slug  string `json:"slug"`
		Color string `json:"color"`
		ID    int    `json:"id"`
	} `json:"tags"`
	Rack struct {
		URL         string `json:"url"`
		Name        string `json:"name"`
		DisplayName string `json:"display_name"`
		ID          int    `json:"id"`
		DeviceCount int    `json:"device_count"`
	} `json:"rack"`
	PrimaryIP4 struct {
		URL     string `json:"url"`
		Address string `json:"address"`
		ID      int    `json:"id"`
		Family  int    `json:"family"`
	} `json:"primary_ip4"`
	PrimaryIP struct {
		URL     string `json:"url"`
		Address string `json:"address"`
		ID      int    `json:"id"`
		Family  int    `json:"family"`
	} `json:"primary_ip"`
	Cluster struct {
		URL                 string `json:"url"`
		Name                string `json:"name"`
		ID                  int    `json:"id"`
		VirtualmachineCount int    `json:"virtualmachine_count"`
	} `json:"cluster"`
	PrimaryIP6 struct {
		URL     string `json:"url"`
		Address string `json:"address"`
		ID      int    `json:"id"`
		Family  int    `json:"family"`
	} `json:"primary_ip6"`
	VirtualChassis struct {
		URL    string `json:"url"`
		Master struct {
			URL         string `json:"url"`
			Name        string `json:"name"`
			DisplayName string `json:"display_name"`
			ID          int    `json:"id"`
		} `json:"master"`
		ID          int `json:"id"`
		MemberCount int `json:"member_count"`
	} `json:"virtual_chassis"`
	DeviceRole struct {
		URL                 string `json:"url"`
		Name                string `json:"name"`
		Slug                string `json:"slug"`
		ID                  int    `json:"id"`
		DeviceCount         int    `json:"device_count"`
		VirtualmachineCount int    `json:"virtualmachine_count"`
	} `json:"device_role"`
	Platform struct {
		URL                 string `json:"url"`
		Name                string `json:"name"`
		Slug                string `json:"slug"`
		ID                  int    `json:"id"`
		DeviceCount         int    `json:"device_count"`
		VirtualmachineCount int    `json:"virtualmachine_count"`
	} `json:"platform"`
	DeviceType struct {
		URL          string `json:"url"`
		Model        string `json:"model"`
		Slug         string `json:"slug"`
		DisplayName  string `json:"display_name"`
		Manufacturer struct {
			URL             string `json:"url"`
			Name            string `json:"name"`
			Slug            string `json:"slug"`
			ID              int    `json:"id"`
			DevicetypeCount int    `json:"devicetype_count"`
		} `json:"manufacturer"`
		ID          int `json:"id"`
		DeviceCount int `json:"device_count"`
	} `json:"device_type"`
	Position   int `json:"position"`
	VcPriority int `json:"vc_priority"`
	VcPosition int `json:"vc_position"`
	ID         int `json:"id"`
}

type NewDevice struct {
	CustomFields     interface{} `json:"custom_fields,omitempty"`
	Face             string      `json:"face,omitempty"`
	LocalContextData string      `json:"local_context_data,omitempty"`
	Comments         string      `json:"comments,omitempty"`
	Status           string      `json:"status,omitempty"`
	Serial           string      `json:"serial,omitempty"`
	AssetTag         string      `json:"asset_tag,omitempty"`
	ParentDevice     struct {
		Name string `json:"name,omitempty"`
	} `json:"parent_device,omitempty"`
	Name string `json:"name,omitempty"`
	Tags []struct {
		Name  string `json:"name"`
		Slug  string `json:"slug"`
		Color string `json:"color"`
	} `json:"tags"`
	Cluster        int `json:"cluster,omitempty"`
	Platform       int `json:"platform,omitempty"`
	Tenant         int `json:"tenant,omitempty"`
	PrimaryIP4     int `json:"primary_ip4,omitempty"`
	PrimaryIP6     int `json:"primary_ip6,omitempty"`
	Site           int `json:"site,omitempty"`
	VirtualChassis int `json:"virtual_chassis,omitempty"`
	VcPosition     int `json:"vc_position,omitempty"`
	VcPriority     int `json:"vc_priority,omitempty"`
	DeviceRole     int `json:"device_role,omitempty"`
	DeviceType     int `json:"device_type,omitempty"`
	Position       int `json:"position,omitempty"`
	Rack           int `json:"rack,omitempty"`
}

type UpdateDevice struct {
	CustomFields interface{} `json:"custom_fields,omitempty"`
	ParentDevice *struct {
		Name string `json:"name,omitempty"`
	} `json:"parent_device,omitempty"`
	Face             string   `json:"face,omitempty"`
	LocalContextData string   `json:"local_context_data,omitempty"`
	Comments         string   `json:"comments,omitempty"`
	Serial           string   `json:"serial,omitempty"`
	AssetTag         string   `json:"asset_tag,omitempty"`
	Status           string   `json:"status,omitempty"`
	Name             string   `json:"name,omitempty"`
	Tags             []string `json:"tags,omitempty"`
	Position         int      `json:"position,omitempty"`
	DeviceType       int      `json:"device_type,omitempty"`
	Site             int      `json:"site,omitempty"`
	PrimaryIP4       int      `json:"primary_ip4,omitempty"`
	PrimaryIP6       int      `json:"primary_ip6,omitempty"`
	Cluster          int      `json:"cluster,omitempty"`
	VirtualChassis   int      `json:"virtual_chassis,omitempty"`
	VcPosition       int      `json:"vc_position,omitempty"`
	VcPriority       int      `json:"vc_priority,omitempty"`
	Platform         int      `json:"platform,omitempty"`
	Tenant           int      `json:"tenant,omitempty"`
	Rack             int      `json:"rack,omitempty"`
	DeviceRole       int      `json:"device_role,omitempty"`
}

// DeviceFilter is used to filter dcim_device_list query to the Netbox API
type DeviceFilter struct {
	// User specific filters
	ID                  string `schema:"id,omitempty"`
	Name                string `schema:"name,omitempty"`
	AssetTag            string `schema:"asset_tag,omitempty"`
	Face                string `schema:"face,omitempty"`
	Position            string `schema:"position,omitempty"`
	VcPosition          string `schema:"vc_position,omitempty"`
	VcPriority          string `schema:"vc_priority,omitempty"`
	TenantGroupID       string `schema:"tenant_group_id,omitempty"`
	TenantGroup         string `schema:"tenant_group,omitempty"`
	TenantID            string `schema:"tenant_id,omitempty"`
	Tenant              string `schema:"tenant,omitempty"`
	IDIn                string `schema:"id__in,omitempty"`
	Q                   string `schema:"q,omitempty"`
	ManufacturerID      string `schema:"manufacturer_id,omitempty"`
	Manufacturer        string `schema:"manufacturer,omitempty"`
	DeviceTypeID        string `schema:"device_type_id,omitempty"`
	RoleID              string `schema:"role_id,omitempty"`
	Role                string `schema:"role,omitempty"`
	RegionID            string `schema:"region_id,omitempty"`
	Region              string `schema:"region,omitempty"`
	SiteID              string `schema:"site_id,omitempty"`
	Site                string `schema:"site,omitempty"`
	RackGroupID         string `schema:"rack_group_id,omitempty"`
	RackID              string `schema:"rack_id,omitempty"`
	ClusterID           string `schema:"cluster_id,omitempty"`
	Model               string `schema:"model,omitempty"`
	Status              string `schema:"status,omitempty"`
	IsfullDepth         string `schema:"is_full_depth,omitempty"`
	MacAddress          string `schema:"mac_address,omitempty"`
	Serial              string `schema:"serial,omitempty"`
	HasPrimaryIP        string `schema:"has_primary_ip,omitempty"`
	VirtualChassiID     string `schema:"virtual_chassis_id,omitempty"`
	VirtualChassiMember string `schema:"virtual_chassis_member,omitempty"`
	ConsolePorts        string `schema:"console_ports,omitempty"`
	ConsoleServerPorts  string `schema:"console_server_ports,omitempty"`
	PowerPorts          string `schema:"power_ports,omitempty"`
	PowerOutlets        string `schema:"power_outlets,omitempty"`
	Interfaces          string `schema:"interfaces,omitempty"`
	PassThroughPorts    string `schema:"pass_through_ports,omitempty"`
	Tag                 string `schema:"tag,omitempty"`

	Offset int `schema:"offset,omitempty"`
	Limit  int `schema:"limit,omitempty"`
}

const devicesPath = dcimPath + "/devices"

// List devices. DeviceFilter is used to list based on filter queries.
func (s *DevicesService) List(ctx context.Context, f *DeviceFilter) (*Devices, error) {
	var devices Devices
	var query string
	var req *http.Request
	var err error

	encoder := schema.NewEncoder()

	query = ""
	if f != nil {
		form := url.Values{}
		err = encoder.Encode(f, form)
		if err != nil {
			return &devices, err
		}
		query = form.Encode()
	}

	req, err = s.client.newRequest(ctx, "GET", devicesPath, query, nil)
	if err != nil {
		return &devices, err
	}

	_, err = s.client.do(req, &devices)
	if err != nil {
		return &devices, err
	}

	return &devices, nil
}

// Create a device
func (s *DevicesService) Create(ctx context.Context, c *NewDevice) error {
	var err error
	var req *http.Request

	req, err = s.client.newRequest(ctx, "POST", devicesPath, "", c)
	if err != nil {
		return fmt.Errorf("unable to create request: %w", err)
	}

	_, err = s.client.do(req, nil)
	if err != nil {
		return fmt.Errorf("unable to do request: %w", err)
	}

	return nil
}

// Update a device
func (s *DevicesService) Update(ctx context.Context, id string, c *UpdateDevice) error {
	var req *http.Request
	var err error

	path := fmt.Sprintf("%s/%s", devicesPath, id)

	req, err = s.client.newRequest(ctx, "PATCH", path, "", c)
	if err != nil {
		return fmt.Errorf("unable to create request: %w", err)
	}

	_, err = s.client.do(req, nil)
	if err != nil {
		return fmt.Errorf("unable to do request: %w", err)
	}

	return nil
}
